@@ -93,13 +93,19 @@ class DataClassMeta(type):
             from functools import total_ordering
             total_ordering(cls)
 
+        # determine a static expression for an instance's fields as a tuple, then evaluate this to create a property
+        # allowing efficient representation for internal methods
+        tuple_expr = f'({", ".join((*(f"self.{f}" for f in fields(cls)), ""))})'  # "" ensures closing comma
+        cls.__tuple__ = property(eval(f'lambda self: {tuple_expr}'))
+
 
 class DataClassInit(DataClassMeta):
     """In the case that a custom __init__ is defined, remove arguments used by __new__ before calling it."""
     def __call__(cls, *args, **kwargs):
-        instance = cls.__new__(cls, *args, **kwargs)
+        args = iter(args)
+        new_kwargs = dict(zip(cls.__annotations__, args))  # convert positional args to keyword for __new__
+        instance = cls.__new__(cls, **new_kwargs, **kwargs)
 
-        args = args[cls.__new__.__code__.co_argcount - 1:]  # -1 for 'cls'
         for parameter in kwargs.keys() & cls.__annotations__.keys():
             del kwargs[parameter]
 
@@ -117,13 +123,17 @@ class DataClassInit(DataClassMeta):
 def _generate_new(annotations: Dict, defaults: Dict, user_init: bool, gen_kwargs: bool, frozen: bool) -> Function:
     """Generate and return a __new__ method for a data class which has as parameters all fields of the data class.
     When the data class is initialised, arguments to this function are applied to the fields of the new instance. Using
-    __new__ frees up __init__, allowing it to be defined by the user to perform additional, custom initialisation."""
+    __new__ frees up __init__, allowing it to be defined by the user to perform additional, custom initialisation.
+
+    If __init__ is defined, all arguments to __new__ become keyword-only, and the custom __call__ converts positional
+    arguments to keyword arguments. This prevents ambiguity during attempts to pass the same argument twice,
+    positionally and as a keyword."""
     arguments = [a for a in annotations if a not in defaults]
     default_arguments = [f'{a}={a}' for a in annotations if a in defaults]
-    args = ['*args'] if user_init else []  # if init is defined, new's arguments must be kw-only to avoid ambiguity
-    kwargs = ['**kwargs'] if gen_kwargs or user_init else []
+    kw_only = ['*'] if user_init and (arguments or default_arguments) else []
+    kwargs = ['**kwargs'] if gen_kwargs or user_init else []  # pass through unknown keyword arguments to __init__
 
-    parameters = ', '.join(arguments + args + default_arguments + kwargs)
+    parameters = ', '.join(kw_only + arguments + default_arguments + kwargs)
 
     # determine what to do with arguments before assignment. If the argument matches a mutable default, make a copy
     references = {n: f'{n}.copy() if {n} is self.__defaults__[{n!r}] else {n}'
@@ -145,25 +155,26 @@ def _generate_new(annotations: Dict, defaults: Dict, user_init: bool, gen_kwargs
 
 # generic method implementations common to all data classes
 # these are currently relatively inefficient - it would be better to cache an expression for a class' tuple
-from .functions import values, as_tuple
+from .functions import values, fields
 
 
 def __eq__(self: DataClass, other: DataClass):
-    return type(self) is type(other) and as_tuple(self) == as_tuple(other)
+    return type(self) is type(other) and self.__tuple__ == other.__tuple__
 
 
 def __lt__(self: DataClass, other: DataClass):
     if isinstance(other, type(self)):
-        return as_tuple(self) < as_tuple(other)
+        return self.__tuple__ < other.__tuple__
     return NotImplemented
 
 
 def __hash__(self):
-    return hash((type(self), *(v for v in as_tuple(self) if v is isinstance(v, Hashable))))
+    # currently not ideal since gives no warning if a field expected to be hashable is not
+    return hash((type(self), *(v for v in self.__tuple__ if v is isinstance(v, Hashable))))
 
 
 def __iter__(self):
-    return iter(as_tuple(self))
+    return iter(self.__tuple__)
 
 
 def __repr__(self):
